@@ -2,7 +2,7 @@
 
 #![no_std]
 
-use core::fmt::{Debug, Formatter};
+use core::fmt::{Debug, Display, Formatter};
 use core::marker::PhantomData;
 use core::ops::{BitAnd, BitOrAssign, BitXorAssign, Shl, ShlAssign, Shr, ShrAssign};
 use num_traits::{Bounded, Num};
@@ -51,6 +51,13 @@ bit_counts!(u8 = 8, u16 = 16, u32 = 32, u64 = 64, u128 = 128, bool = 1);
 impl u1 {
     pub const TRUE: u1 = u1(1);
     pub const FALSE: u1 = u1(0);
+
+    pub fn toggle(self) -> Self {
+        match self {
+            Self::FALSE => u1::TRUE,
+            _ => Self::FALSE
+        }
+    }
 }
 
 macro_rules! new_types {
@@ -69,6 +76,12 @@ macro_rules! new_types {
             }
         }
 
+        impl Display for $name {
+            fn fmt(&self, f: &mut Formatter<'_>) -> core::fmt::Result {
+                f.write_fmt(format_args!("{}", self.0))
+            }
+        }
+
         #[macro_export]
         macro_rules! $name {
             ($value: literal) => {
@@ -78,7 +91,7 @@ macro_rules! new_types {
                     // this is always valid because we have one more bit than we need in $inner
                     // type
                     const MAX: $inner = (1 << ($count)) - 1;
-                    const PANIC_IF_GT: $inner = MAX - VALUE;
+                    const _: () = assert!(MAX >= VALUE, "The provided value is too large");
                     unsafe {bit_struct::$name::new_unchecked(VALUE)}
                 }
             };
@@ -340,11 +353,7 @@ macro_rules! bit_struct_impl {
 
         impl Default for $name {
             fn default() -> Self {
-                let mut res = unsafe { Self::from_unchecked(0) };
-                $(
-                    res.$field().set(Default::default());
-                )*
-                res
+                Self::of_defaults()
             }
         }
 
@@ -360,6 +369,21 @@ macro_rules! bit_struct_impl {
         }
     ) => {
 
+        impl $name {
+
+            /// Returns a valid representation for the bit_struct, where all values are
+            /// the defaults. This is different than Self::default(), because the actual
+            /// default implementation might not be composed of only the defaults of the
+            /// given fields
+            pub fn of_defaults() -> Self {
+                let mut res = unsafe { Self::from_unchecked(0) };
+                $(
+                    res.$field().set(Default::default());
+                )*
+                res
+            }
+        }
+
         impl ::core::fmt::Debug for $name {
             fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> core::fmt::Result {
                 let mut copied = *self;
